/*=========================================================================                                                                               
  Program:   bbtk
  Module:    $RCSfile: bbtkAtomicBlackBoxInputDescriptor.h,v $
  Language:  C++
  Date:      $Date: 2008/10/17 08:18:12 $
  Version:   $Revision: 1.3 $
=========================================================================*/

/* ---------------------------------------------------------------------

* Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
* Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
*
*  This software is governed by the CeCILL-B license under French law and 
*  abiding by the rules of distribution of free software. You can  use, 
*  modify and/ or redistribute the software under the terms of the CeCILL-B 
*  license as circulated by CEA, CNRS and INRIA at the following URL 
*  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
*  or in the file LICENSE.txt.
*
*  As a counterpart to the access to the source code and  rights to copy,
*  modify and redistribute granted by the license, users are provided only
*  with a limited warranty  and the software's author,  the holder of the
*  economic rights,  and the successive licensors  have only  limited
*  liability. 
*
*  The fact that you are presently reading this means that you have had
*  knowledge of the CeCILL-B license and that you accept its terms.
* ------------------------------------------------------------------------ */                                                                         


/**
 *  \file 
 *  \brief Class bbtk::AtomicBlackBoxInputDescriptor : structure containing the description of a AtomicBlackBox input (name, description, type, functor)
 */
/**
 * \class bbtk::AtomicBlackBoxInputDescriptor
 * \brief Structure containing the description of a AtomicBlackBox input (name, description, type, functor)
 */
 
#ifndef __bbtkAtomicBlackBoxInputDescriptor_h__
#define __bbtkAtomicBlackBoxInputDescriptor_h__

#include "bbtkBlackBoxInputDescriptor.h"
#include "bbtkAtomicBlackBoxGetSetFunctor.h"
#include <typeinfo>

namespace bbtk
{


  class BBTK_EXPORT AtomicBlackBoxInputDescriptor : 
    public bbtk::BlackBoxInputDescriptor
  {
  public:
    //typedef enum {
    //  MANDATORY,
    //  OPTIONAL
    //} InputType;

      
    AtomicBlackBoxInputDescriptor( TypeInfo creator_type_info,
				   const std::string& name,
				   const std::string& description,
				   const std::string& nature,
				   AtomicBlackBoxGetFunctor* getfunctor,
				   AtomicBlackBoxSetFunctor* setfunctor,
				   bool copy_construct = true);
    
    ~AtomicBlackBoxInputDescriptor();  
    /// Returns the type of the input
    TypeInfo GetTypeInfo() const { return mGetFunctor->GetTypeInfo(); }
    /// Returns the name of the type of the input
    std::string GetTypeName() const { return mGetFunctor->GetTypeName(); }
    /// Returns the name of the type of the input
    std::string GetHumanTypeName() const { return mGetFunctor->GetHumanTypeName(); }    
    /// Returns true iff the type is a pointer to class 
    virtual bool IsPointerType() const { return mGetFunctor->IsPointerType(); }    
    /// Returns the functor on the Get method
    AtomicBlackBoxGetFunctor* GetGetFunctor() { return mGetFunctor; }
    /// Returns the functor on the Set method
    AtomicBlackBoxSetFunctor* GetSetFunctor() { return mSetFunctor; }
    
    
  private:
    /// Default ctor is private 
    AtomicBlackBoxInputDescriptor() : 
      BlackBoxInputDescriptor(typeid(void),"","","") {}
    /// The functor on the Get method
    AtomicBlackBoxGetFunctor* mGetFunctor;
    /// The functor on the Set method
    AtomicBlackBoxSetFunctor* mSetFunctor;
  };
  
}
// namespace bbtk
#endif
