/*=========================================================================                                                                               
  Program:   bbtk
  Module:    $RCSfile: bbtkBlackBoxInputDescriptor.h,v $
  Language:  C++
  Date:      $Date: 2008/10/17 08:18:12 $
  Version:   $Revision: 1.5 $
=========================================================================*/

/* ---------------------------------------------------------------------

* Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
* Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
*
*  This software is governed by the CeCILL-B license under French law and 
*  abiding by the rules of distribution of free software. You can  use, 
*  modify and/ or redistribute the software under the terms of the CeCILL-B 
*  license as circulated by CEA, CNRS and INRIA at the following URL 
*  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
*  or in the file LICENSE.txt.
*
*  As a counterpart to the access to the source code and  rights to copy,
*  modify and redistribute granted by the license, users are provided only
*  with a limited warranty  and the software's author,  the holder of the
*  economic rights,  and the successive licensors  have only  limited
*  liability. 
*
*  The fact that you are presently reading this means that you have had
*  knowledge of the CeCILL-B license and that you accept its terms.
* ------------------------------------------------------------------------ */                                                                         


/**
 *  \file 
 *  \brief Class bbtk::BlackBoxInputDescriptor : abstract descriptor of an input of a black box 
 */
/**
 * \class bbtk::BlackBoxInputDescriptor
 * \brief Abstract descriptor of an input of a black box 
 */

#ifndef __bbtkBlackBoxInputDescriptor_h__
#define __bbtkBlackBoxInputDescriptor_h__

#include "bbtkBlackBoxInputOutputDescriptor.h"

namespace bbtk
{
  class BBTK_EXPORT BlackBoxInputDescriptor 
    : public BlackBoxInputOutputDescriptor
  {
  public:
    typedef enum {
      MANDATORY,
      OPTIONAL
    } OptionType;
    
    
    /// Ctor with name and description
    BlackBoxInputDescriptor(TypeInfo creator_type_info,
			    const std::string& name,
			    const std::string& description,
			    const std::string& nature,
			    bool copy_construct = true,
			    OptionType option = MANDATORY
			    )
      : BlackBoxInputOutputDescriptor(creator_type_info,name,
				      description,nature,
				      copy_construct),
	mOption(option)
    {}
    /// Returns the option of the input
    OptionType GetOption() const { return mOption; }
    
  protected:
    /// Default ctor is reserved to children
    BlackBoxInputDescriptor() : BlackBoxInputOutputDescriptor(),
				mOption(OPTIONAL) {}

  private:
    /// The option of the input 
    OptionType mOption;
  };
  
}
// namespace bbtk
#endif
