/*=========================================================================                                                                               
  Program:   bbtk
  Module:    $RCSfile: bbtkQt.h,v $
  Language:  C++
  Date:      $Date: 2010/01/14 13:17:27 $
  Version:   $Revision: 1.2 $
=========================================================================*/

/* ---------------------------------------------------------------------
* Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
* Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
*
*  This software is governed by the CeCILL-B license under French law and 
*  abiding by the rules of distribution of free software. You can  use, 
*  modify and/ or redistribute the software under the terms of the CeCILL-B 
*  license as circulated by CEA, CNRS and INRIA at the following URL 
*  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
*  or in the file LICENSE.txt.
*
*  As a counterpart to the access to the source code and  rights to copy,
*  modify and redistribute granted by the license, users are provided only
*  with a limited warranty  and the software's author,  the holder of the
*  economic rights,  and the successive licensors  have only  limited
*  liability. 
*
*  The fact that you are presently reading this means that you have had
*  knowledge of the CeCILL-B license and that you accept its terms.
* ------------------------------------------------------------------------ */                                                                         

#ifndef __bbtkQt_h_INCLUDED__
#define __bbtkQt_h_INCLUDED__

//===========================================================================
// SIGNAL/SLOT MECHANISM 
// FOR bbtk WINDOWS CREATION/DESTRUCTION OBSERVATION 
// BBTKWXSIG
#include <boost/signal.hpp>
#include <boost/bind.hpp>
typedef boost::signals::trackable SignalObserver;
// \BBTKWXSIG
//===========================================================================


//===========================================================================
// Qt headers
#ifdef USE_QT
#include "qwidget.h"
// EO Qt headers

#else //USE_QT

// define QWidget
typedef void QWidget;
typedef void QBusyCursor;

#endif // EO USE_WXWIDGETS
//===========================================================================

//===========================================================================
#include "bbtkSystem.h"
//===========================================================================


namespace bbtk
{

  //==================================================================
  /// Global qt handlers (cursor...)
  
// BBTKWXSIG
#define bbtkAddQtSignalObserver(METHOD)		\
  Qt::AddSignalObserver(boost::bind(&METHOD, this))
// \BBTKWXSIG

#define BBTK_BUSY_CURSOR bbtk::Qt::BusyCursor __bbtk_dummy_busy_cursor;

  struct BBTK_EXPORT Qt
  {	
  public:
    // BBTKWXSIG
    // The keyword 'signals' is also a macro in qt
    //    typedef boost::signals::trackable SignalObserver;
    typedef boost::signal<void ()>  Signal_type;
    typedef Signal_type::slot_function_type Slot_function_type;
    // \BBTKWXSIG

    /*
    static void ResetCursor();
    static void BeginBusyCursor(); 
    static void EndBusyCursor();
    */

    static void LoopUntilAllWindowsClose();

    class BusyCursor
    {
    public:
      BusyCursor();
      ~BusyCursor();
      //wxBusyCursor* mCursor;
    };

    static void SetTopWindowParent(QWidget*);
    static void SetAutoDestroyTopWindow(bool);

    /// Returns the creation time parent of all bbtk windows 
    /// (window can be reparented after creation)
    static QWidget* GetTopWindow();
    static bool TopWindowExists();
    
    static void IncNbWindowsAlive();
    static void DecNbWindowsAlive();
    static int  GetNbWindowsAlive();
    static bool IsSomeWindowAlive();
    
    static void IncNbWindowsShown();
    static void DecNbWindowsShown();
    static int  GetNbWindowsShown();
    static bool IsSomeWindowShown();

    // BBTKWXSIG
    static void AddSignalObserver(Slot_function_type);
    // \BBTKWXSIG



    static void CreateQtAppIfNeeded();
    static void DestroyQtAppIfNeeded();

  private:
    static void CreateTopWindowIfNeeded();
    static void DestroyTopWindowIfNeeded();

    /// Sets the creation time parent of all bbtk windows
    static void SetTopWindow(QWidget*);
  }; // struct Qt
  //==================================================================
  




  
} // namespace bbtk
  

#endif // EO __bbtkQt_h_INCLUDED__
